/*
 * i386/jit.h
 * Common i386 JIT configuration information.
 *
 * Copyright (c) 1996 Systems Architecture Research Centre,
 *		   City University, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@sarc.city.ac.uk>, June 1996.
 */

#ifndef __i386_jit_h
#define __i386_jit_h

/**/
/* Exception handling information. */
/**/

extern struct _thread* currentThread;

/* Structure of exception frame on stack */
typedef struct _exceptionFrame {
        uintp	retbp;
        uintp	retpc;
} exceptionFrame;

/* Is this frame valid (ie. is it on the current stack) ? */
#define	FRAMEOKAY(f)							\
	((f)->retbp >= (int)currentThread->PrivateInfo->stackBase &&	\
	 (f)->retbp < (int)currentThread->PrivateInfo->stackEnd)

/* Get the next frame in the chain */
#define	NEXTFRAME(f)							\
	((exceptionFrame*)(f)->retbp)

/* Extract the PC from the given frame */
#define	PCFRAME(f)							\
	((f)->retpc-1)

/* Get the first exception frame from a subroutine call */
#define	FIRSTFRAME(f, o)						\
	(f) = *((exceptionFrame*)(((uintp)&(o))-8))

/* Call the relevant exception handler (rewinding the stack as
   necessary). */
#define CALL_KAFFE_EXCEPTION(frame, info, obj)				\
        asm("   pushl %2						\n\
		movl %0,%%ebp						\n\
		jmpl *%1						\n\
		" : : "g" (frame->retbp), "r" (info.handler), "g" (obj))

/**/
/* Register management information. */
/**/

/* Define the register set */
#define	REGISTER_SET							\
	{ /* eax */	0, Rint|Rref,	0, 0    },			\
	{ /* ecx */	0, Rint|Rref,	0, 0    },			\
	{ /* edx */	0, Rint|Rref,	0, 0    },			\
	{ /* ebx */	0, Rint|Rref,	0, 0    },			\
	{ /* esp */	0, Reserved,	0, 0    },			\
	{ /* ebp */	0, Reserved,	0, 0    },			\
	{ /* esi */	0, Reserved,	0, 0    },			\
	{ /* edi */	0, Reserved,	0, 0    },			\
	{ /* f0  */	0, Rfloat,	1, 0    },			\
	{ /* d0  */	0, Rdouble,	1, 0    },

/* Number of registers in the register set */
#define	NR_REGISTERS	10

/**/
/* Slot management information. */
/**/

/* Size of each slot */
#define	SLOTSIZE	4

/* Generate slot offset for an argument */
#define SLOT2ARGOFFSET(_n)	(8 + SLOTSIZE * (_n))

/* Generate slot offset for a local (non-argument) */
#define SLOT2LOCALOFFSET(_n)	(-SLOTSIZE * (maxTemp+maxLocal+maxStack - (_n)))

#endif
