/* basecode.c
 *
 * Copyright (c) 1996 Systems Architecture Research Centre,
 *		   City University, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@sarc.city.ac.uk>, June 1996.
 */

#include "gtypes.h"
#include "slots.h"
#include "seq.h"
#include "registers.h"
#include "icode.h"
#include "basecode.h"

#define	SLOTDEBUG

void
_cancel_insn(slots* dst)
{
#ifdef notyet
	if (dst->insn != 0) {
		dst->insn->func = 0;
	}
#endif
}

void
_slot_const_const(slots* dst, int s1, int s2, ifunc f, int type)
{
	seq->u[1].iconst = s1;
	seq->u[2].iconst = s2;

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->write = ++writecount;
		dst->type = type;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;
}

void
_slot_slot_const(slots* dst, slots* s1, int s2, ifunc f, int type)
{
	if (s1 != 0) {
		while (slot_copy(s1)) {
			_cancel_insn(s1);
			s1 = slot_where(s1);
		}
		seq->u[1].slot = s1;
		s1->read = s1->write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[1].slot = 0;
	}
#endif

	seq->u[2].iconst = s2;

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->write = ++writecount;
		dst->type = type;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;
}

void
_slot_slot_fconst(slots* dst, slots* s1, double s2, ifunc f, int type)
{
	if (s1 != 0) {
		while (slot_copy(s1)) {
			_cancel_insn(s1);
			s1 = slot_where(s1);
		}
		seq->u[1].slot = s1;
		s1->read = s1->write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[1].slot = 0;
	}
#endif

	seq->u[2].fconst = s2;

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->write = ++writecount;
		dst->type = type;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;
}

void
_slot_slot_slot(slots* dst, slots* s1, slots* s2, ifunc f, int type)
{
#if defined(TWO_OPERAND)
	/* Two operand systems cannot handle three operand ops.
	 * We need to fixit so the dst is one of the source ops.
	 */
	slots* olddst = 0;
	if (s1 != 0 && s2 != 0 && dst != 0) {
		if (s2 == dst) {
			olddst = dst;
			slot_alloctmp(dst);
		}
		if (s1 != dst) {
			move_any(dst, s1);
			s1 = dst;
		}
	}
#endif

	if (s1 != 0) {
		while (slot_copy(s1)) {
			_cancel_insn(s1);
			s1 = slot_where(s1);
		}
		seq->u[1].slot = s1;
		s1->read = s1->write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[1].slot = 0;
	}
#endif

	if (s2 != 0) {
		while (slot_copy(s2)) {
			_cancel_insn(s2);
			s2 = slot_where(s2);
		}
		seq->u[2].slot = s2;
		s2->read = s2->write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[2].slot = 0;
	}
#endif

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->type = type;
		if (type == Tcopy) {
			dst->where = s2;
			dst->write = s2->write;
		}
		else {
			dst->write = ++writecount;
		}
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;

#if defined(TWO_OPERAND)
	if (olddst != 0) {
		move_any(olddst, dst);
	}
#endif
}

void
_lslot_lslot_lslot(slots* dst, slots* s1, slots* s2, ifunc f, int type)
{
#if defined(TWO_OPERAND)
	/* Two operand systems cannot handle three operand ops.
	 * We need to fixit so the dst is one of the source ops.
	 */
	slots* olddst = 0;
	if (s1 != 0 && s2 != 0 && dst != 0) {
		if (s2 == dst) {
			olddst = dst;
			slot_alloc2tmp(dst);
		}
		if (s1 != dst) {
			move_anylong(dst, s1);
			s1 = dst;
		}
	}
#endif

	if (s1 != 0) {
		while (slot_copy(s1)) {
			_cancel_insn(s1);
			s1 = slot_where(s1);
		}
		seq->u[1].slot = s1;
		s1[0].read = s1[0].write;
		s1[1].read = s1[1].write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[1].slot = 0;
	}
#endif

	if (s2 != 0) {
		while (slot_copy(s2)) {
			_cancel_insn(s2);
			s2 = slot_where(s2);
		}
		seq->u[2].slot = s2;
		s2[0].read = s2[0].write;
		s2[1].read = s2[1].write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[2].slot = 0;
	}
#endif

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->type = type;
		if (type == Tcopy) {
			dst[0].where = &s2[0];
			dst[1].where = &s2[1];
			dst[0].write = s2[0].write;
			dst[1].write = s2[1].write;
		}
		else {
			dst[0].write = ++writecount;
			dst[1].write = ++writecount;
		}
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;

#if defined(TWO_OPERAND)
	if (olddst != 0) {
		move_anylong(olddst, dst);
	}
#endif
}

void
_lslot_lslot_slot(slots* dst, slots* s1, slots* s2, ifunc f, int type)
{
#if defined(TWO_OPERAND)
	/* Two operand systems cannot handle three operand ops.
	 * We need to fixit so the dst is one of the source ops.
	 */
	slots* olddst = 0;
	if (s1 != 0 && s2 != 0 && dst != 0) {
		if (s2 == dst) {
			olddst = dst;
			slot_alloctmp(dst);
		}
		if (s1 != dst) {
			move_any(dst, s1);
			s1 = dst;
		}
	}
#endif

	if (s1 != 0) {
		while (slot_copy(s1)) {
			_cancel_insn(s1);
			s1 = slot_where(s1);
		}
		seq->u[1].slot = s1;
		s1[0].read = s1[0].write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[1].slot = 0;
	}
#endif

	if (s2 != 0) {
		while (slot_copy(s2)) {
			_cancel_insn(s2);
			s2 = slot_where(s2);
		}
		seq->u[2].slot = s2;
		s2[0].read = s2[0].write;
		s2[1].read = s2[1].write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[2].slot = 0;
	}
#endif

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->type = type;
		if (type == Tcopy) {
			dst->where = s2;
			dst[0].write = s2[0].write;
			dst[1].write = s2[1].write;
		}
		else {
			dst[0].write = ++writecount;
			dst[1].write = ++writecount;
		}
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;

#if defined(TWO_OPERAND)
	if (olddst != 0) {
		move_any(olddst, dst);
	}
#endif
}

void
_slot_slot_lslot(slots* dst, slots* s1, slots* s2, ifunc f, int type)
{
#if defined(TWO_OPERAND)
	/* Two operand systems cannot handle three operand ops.
	 * We need to fixit so the dst is one of the source ops.
	 */
	slots* olddst = 0;
	if (s1 != 0 && s2 != 0 && dst != 0) {
		if (s2 == dst) {
			olddst = dst;
			slot_alloctmp(dst);
		}
		if (s1 != dst) {
			move_any(dst, s1);
			s1 = dst;
		}
	}
#endif

	if (s1 != 0) {
		while (slot_copy(s1)) {
			_cancel_insn(s1);
			s1 = slot_where(s1);
		}
		seq->u[1].slot = s1;
		s1[0].read = s1[0].write;
		s1[1].read = s1[1].write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[1].slot = 0;
	}
#endif

	if (s2 != 0) {
		while (slot_copy(s2)) {
			_cancel_insn(s2);
			s2 = slot_where(s2);
		}
		seq->u[2].slot = s2;
		s2->read = s2->write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[2].slot = 0;
	}
#endif

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->type = type;
		if (type == Tcopy) {
			dst->where = s2;
			dst->write = s2->write;
		}
		else {
			dst->write = ++writecount;
		}
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;

#if defined(TWO_OPERAND)
	if (olddst != 0) {
		move_any(olddst, dst);
	}
#endif
}

void
_lslot_slot_lconst(slots* dst, slots* s1, jlong s2, ifunc f, int type)
{
	if (s1 != 0) {
		while (slot_copy(s1)) {
			_cancel_insn(s1);
			s1 = slot_where(s1);
		}
		seq->u[1].slot = s1;
		s1[0].read = s1[0].write;
		s1[1].read = s1[1].write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[1].slot = 0;
	}
#endif

	seq->u[2].lconst = s2;

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->type = type;
		dst[0].write = ++writecount;
		dst[1].write = ++writecount;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;
}

void
_lslot_slot_fconst(slots* dst, slots* s1, double s2, ifunc f, int type)
{
	if (s1 != 0) {
		while (slot_copy(s1)) {
			_cancel_insn(s1);
			s1 = slot_where(s1);
		}
		seq->u[1].slot = s1;
		s1[0].read = s1[0].write;
		s1[1].read = s1[1].write;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[1].slot = 0;
	}
#endif

	seq->u[2].fconst = s2;

	if (dst != 0) {
		if (dst->write > dst->read) {
			_cancel_insn(dst);
		}
		seq->u[0].slot = dst;
		dst->insn = seq;
		dst->type = type;
		dst[0].write = ++writecount;
		dst[1].write = ++writecount;
	}
#if defined(SLOTDEBUG)
	else {
		seq->u[0].slot = 0;
	}
#endif

	seq->func = f;
	seq++;
}
