/* labels.c
 * Manage the code labels and links.
 *
 * Copyright (c) 1996 Systems Architecture Research Centre,
 *		   City University, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@sarc.city.ac.uk>, June 1996.
 */

#include <assert.h>
#include "gtypes.h"
#include "labels.h"
#include "codeinfo.h"

extern uintp realcodediff;

label labels[MAXLABELS];
int lab = 0;

void
linkLabels(void)
{
	int i;
	uintp dest;
	uintp from;
	uintp* place;

	for (i = 0; i < lab; i++) {

		if ((labels[i].type & Llong) != Llong) {
			printf("Label not long (%x).\n", &labels[i]);
			abort();
		}

		/* Find destination of jump */
		if (labels[i].type & Lcode) {
			assert(code[labels[i].to].pc != (uintp)-1);
			dest = code[labels[i].to].pc;
		}
		else {
			dest = labels[i].to;
		}

		/* If internal destination, adjust it */
		if (labels[i].type & Linternal) {
			dest += realcodediff;
		}

		/* Find place to store result */
		place = (uintp*)(labels[i].at + realcodediff);

		/* If label is relative, find where it is relative to */
		if (labels[i].type & Lrelative) {
			from = labels[i].from + realcodediff;
		}
		else {
			from = 0;
		}

		/* Install result */
		(*place) = dest - from;
	}

	lab = 0;
}
